from mokaplayer.ui.gtk.windows.about_window import AboutWindow
from mokaplayer.ui.gtk.windows.help_shortcuts_window import HelpShortcutsWindow
from mokaplayer.ui.gtk.windows.inputbox_window import InputBox
from mokaplayer.ui.gtk.windows.lyrics_window import LyricsWindow
from mokaplayer.ui.gtk.windows.tabs_window import TabsWindow
from mokaplayer.ui.gtk.windows.tagseditor_window import TagsEditorWindow
from mokaplayer.ui.gtk.windows.main_window import MainWindow
